@@ -235,9 +235,9 @@ ExtractWinRoot(
 	if ((path[0] == 'c' || path[0] == 'C')
 		&& (path[1] == 'o' || path[1] == 'O')) {
 	    if ((path[2] == 'm' || path[2] == 'M')
-		    && path[3] >= '1' && path[3] <= '4') {
+		    && path[3] >= '1' && path[3] <= '9') {
 		/*
-		 * May have match for 'com[1-4]:?', which is a serial port.
+		 * May have match for 'com[1-9]:?', which is a serial port.
 		 */
 
 		if (path[4] == '\0') {
@@ -257,9 +257,9 @@ ExtractWinRoot(
 	} else if ((path[0] == 'l' || path[0] == 'L')
 		&& (path[1] == 'p' || path[1] == 'P')
 		&& (path[2] == 't' || path[2] == 'T')) {
-	    if (path[3] >= '1' && path[3] <= '3') {
+	    if (path[3] >= '1' && path[3] <= '9') {
 		/*
-		 * May have match for 'lpt[1-3]:?'
+		 * May have match for 'lpt[1-9]:?'
 		 */
 
 		if (path[4] == '\0') {
@@ -821,10 +821,10 @@ Tcl_FSJoinToPath(
 	return TclJoinPath(2, pair);
     } else {
 	int elemc = objc + 1;
-	Tcl_Obj *ret, **elemv = ckalloc(elemc*sizeof(Tcl_Obj **));
+	Tcl_Obj *ret, **elemv = ckalloc(elemc*sizeof(Tcl_Obj *));
 
 	elemv[0] = pathPtr;
-	memcpy(elemv+1, objv, objc*sizeof(Tcl_Obj **));
+	memcpy(elemv+1, objv, objc*sizeof(Tcl_Obj *));
 	ret = TclJoinPath(elemc, elemv);
 	ckfree(elemv);
 	return ret;
