@@ -126,7 +126,7 @@ typedef struct Channel {
  */
 
 typedef struct ChannelState {
-    const char *channelName;	/* The name of the channel instance in Tcl
+    char *channelName;		/* The name of the channel instance in Tcl
 				 * commands. Storage is owned by the generic
 				 * IO code, is dynamically allocated. */
     int	flags;			/* ORed combination of the flags defined
