@@ -196,6 +196,9 @@ InfoObjectClassCmd(
 	}
 
 	FOREACH(mixinPtr, oPtr->mixins) {
+	    if (!mixinPtr) {
+		continue;
+	    }
 	    if (TclOOIsReachable(o2clsPtr, mixinPtr)) {
 		Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
 		return TCL_OK;
@@ -401,7 +404,7 @@ InfoObjectIsACmd(
 	IsClass, IsMetaclass, IsMixin, IsObject, IsType
     };
     Object *oPtr, *o2Ptr;
-    int idx, i;
+    int idx, i, result = 0;
 
     if (objc < 3) {
 	Tcl_WrongNumArgs(interp, 1, objv, "category objName ?arg ...?");
@@ -412,93 +415,88 @@ InfoObjectIsACmd(
 	return TCL_ERROR;
     }
 
-    if (idx == IsObject) {
-	int ok = (Tcl_GetObjectFromObj(interp, objv[2]) != NULL);
-
-	if (!ok) {
-	    Tcl_ResetResult(interp);
-	}
-	Tcl_SetObjResult(interp, Tcl_NewIntObj(ok ? 1 : 0));
-	return TCL_OK;
-    }
-    oPtr = (Object *) Tcl_GetObjectFromObj(interp, objv[2]);
-    if (oPtr == NULL) {
-	return TCL_ERROR;
-    }
+    /*
+     * Now we know what test we are doing, we can check we've got the right
+     * number of arguments.
+     */
 
     switch ((enum IsACats) idx) {
+    case IsObject:
     case IsClass:
-	if (objc != 3) {
-	    Tcl_WrongNumArgs(interp, 2, objv, "objName");
-	    return TCL_ERROR;
-	}
-	Tcl_SetObjResult(interp, Tcl_NewIntObj(oPtr->classPtr ? 1 : 0));
-	return TCL_OK;
     case IsMetaclass:
 	if (objc != 3) {
 	    Tcl_WrongNumArgs(interp, 2, objv, "objName");
 	    return TCL_ERROR;
 	}
-	if (oPtr->classPtr == NULL) {
-	    Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
-	} else {
-	    Class *classCls = TclOOGetFoundation(interp)->classCls;
-
-	    Tcl_SetObjResult(interp, Tcl_NewIntObj(
-		    TclOOIsReachable(classCls, oPtr->classPtr) ? 1 : 0));
-	}
-	return TCL_OK;
+	break;
     case IsMixin:
-	if (objc != 4) {
-	    Tcl_WrongNumArgs(interp, 2, objv, "objName className");
-	    return TCL_ERROR;
-	}
-	o2Ptr = (Object *) Tcl_GetObjectFromObj(interp, objv[3]);
-	if (o2Ptr == NULL) {
-	    return TCL_ERROR;
-	}
-	if (o2Ptr->classPtr == NULL) {
-	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		    "non-classes cannot be mixins", -1));
-	    Tcl_SetErrorCode(interp, "TCL", "OO", "NONCLASS", NULL);
-	    return TCL_ERROR;
-	} else {
-	    Class *mixinPtr;
-
-	    FOREACH(mixinPtr, oPtr->mixins) {
-		if (mixinPtr == o2Ptr->classPtr) {
-		    Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
-		    return TCL_OK;
-		}
-	    }
-	}
-	Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
-	return TCL_OK;
     case IsType:
 	if (objc != 4) {
 	    Tcl_WrongNumArgs(interp, 2, objv, "objName className");
 	    return TCL_ERROR;
 	}
+	break;
+    }
+
+    /*
+     * Perform the check. Note that we can guarantee that we will not fail
+     * from here on; "failures" result in a false-TCL_OK result.
+     */
+
+    oPtr = (Object *) Tcl_GetObjectFromObj(interp, objv[2]);
+    if (oPtr == NULL) {
+	goto failPrecondition;
+    }
+
+    switch ((enum IsACats) idx) {
+    case IsObject:
+	result = 1;
+	break;
+    case IsClass:
+	result = (oPtr->classPtr != NULL);
+	break;
+    case IsMetaclass:
+	if (oPtr->classPtr != NULL) {
+	    result = TclOOIsReachable(TclOOGetFoundation(interp)->classCls,
+		    oPtr->classPtr);
+	}
+	break;
+    case IsMixin:
 	o2Ptr = (Object *) Tcl_GetObjectFromObj(interp, objv[3]);
 	if (o2Ptr == NULL) {
-	    return TCL_ERROR;
+	    goto failPrecondition;
 	}
-	if (o2Ptr->classPtr == NULL) {
-	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		    "non-classes cannot be types", -1));
-	    Tcl_SetErrorCode(interp, "TCL", "OO", "NONCLASS", NULL);
-	    return TCL_ERROR;
+	if (o2Ptr->classPtr != NULL) {
+	    Class *mixinPtr;
+
+	    FOREACH(mixinPtr, oPtr->mixins) {
+		if (!mixinPtr) {
+		    continue;
+		}
+		if (TclOOIsReachable(o2Ptr->classPtr, mixinPtr)) {
+		    result = 1;
+		    break;
+		}
+	    }
 	}
-	if (TclOOIsReachable(o2Ptr->classPtr, oPtr->selfCls)) {
-	    Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
-	} else {
-	    Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
+	break;
+    case IsType:
+	o2Ptr = (Object *) Tcl_GetObjectFromObj(interp, objv[3]);
+	if (o2Ptr == NULL) {
+	    goto failPrecondition;
 	}
-	return TCL_OK;
-    case IsObject:
-	Tcl_Panic("unexpected fallthrough");
+	if (o2Ptr->classPtr != NULL) {
+	    result = TclOOIsReachable(o2Ptr->classPtr, oPtr->selfCls);
+	}
+	break;
     }
-    return TCL_ERROR;
+    Tcl_SetObjResult(interp, Tcl_NewBooleanObj(result));
+    return TCL_OK;
+
+  failPrecondition:
+    Tcl_ResetResult(interp);
+    Tcl_SetObjResult(interp, Tcl_NewBooleanObj(0));
+    return TCL_OK;
 }
 
 /*
@@ -673,6 +671,9 @@ InfoObjectMixinsCmd(
 
     resultObj = Tcl_NewObj();
     FOREACH(mixinPtr, oPtr->mixins) {
+	if (!mixinPtr) {
+	    continue;
+	}
 	Tcl_ListObjAppendElement(NULL, resultObj,
 		TclOOObjectName(interp, mixinPtr->thisPtr));
     }
@@ -1289,6 +1290,9 @@ InfoClassMixinsCmd(
 
     resultObj = Tcl_NewObj();
     FOREACH(mixinPtr, clsPtr->mixins) {
+	if (!mixinPtr) {
+	    continue;
+	}
 	Tcl_ListObjAppendElement(NULL, resultObj,
 		TclOOObjectName(interp, mixinPtr->thisPtr));
     }
