@@ -995,12 +995,19 @@ TclWinCPUID(
 
     /* See: <http://en.wikipedia.org/wiki/CPUID> */
 #if defined(HAVE_CPUID)
-    __asm__ __volatile__("mov %%ebx, %%edi     \n\t" /* save %ebx */
+#if defined(__x86_64__) || defined(_M_AMD64) || defined (_M_X64)
+    __asm__ __volatile__("movq %%rbx, %%rsi     \n\t" /* save %rbx */
                  "cpuid            \n\t"
-                 "mov %%ebx, %%esi   \n\t" /* save what cpuid just put in %ebx */
-                 "mov %%edi, %%ebx  \n\t" /* restore the old %ebx */
+                 "xchgq %%rsi, %%rbx   \n\t" /* restore the old %rbx */
                  : "=a"(regsPtr[0]), "=S"(regsPtr[1]), "=c"(regsPtr[2]), "=d"(regsPtr[3])
-                 : "a"(index) : "edi");
+                 : "a"(index));
+#else
+    __asm__ __volatile__("mov %%ebx, %%esi     \n\t" /* save %ebx */
+                 "cpuid            \n\t"
+                 "xchg %%esi, %%ebx   \n\t" /* restore the old %ebx */
+                 : "=a"(regsPtr[0]), "=S"(regsPtr[1]), "=c"(regsPtr[2]), "=d"(regsPtr[3])
+                 : "a"(index));
+#endif
     status = TCL_OK;
 #endif
     return status;
