@@ -87,7 +87,7 @@ typedef struct TimeInfo {
 } TimeInfo;
 
 static TimeInfo timeInfo = {
-    { NULL },
+    { NULL, 0, 0, NULL, NULL, 0 },
     0,
     0,
     (HANDLE) NULL,
@@ -156,7 +156,7 @@ TclpGetSeconds(void)
 {
     Tcl_Time t;
 
-    (*tclGetTimeProcPtr) (&t, tclTimeClientData);    /* Tcl_GetTime inlined. */
+    tclGetTimeProcPtr(&t, tclTimeClientData);	/* Tcl_GetTime inlined. */
     return t.sec;
 }
 
@@ -190,42 +190,13 @@ TclpGetClicks(void)
     Tcl_Time now;		/* Current Tcl time */
     unsigned long retval;	/* Value to return */
 
-    (*tclGetTimeProcPtr) (&now, tclTimeClientData);   /* Tcl_GetTime inlined */
+    tclGetTimeProcPtr(&now, tclTimeClientData);	/* Tcl_GetTime inlined */
 
     retval = (now.sec * 1000000) + now.usec;
     return retval;
 
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclpGetTimeZone --
- *
- *	Determines the current timezone. The method varies wildly between
- *	different Platform implementations, so its hidden in this function.
- *
- * Results:
- *	Minutes west of GMT.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TclpGetTimeZone(
-    unsigned long currentTime)
-{
-    int timeZone;
-
-    tzset();
-    timeZone = timezone / 60;
-
-    return timeZone;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -252,7 +223,7 @@ void
 Tcl_GetTime(
     Tcl_Time *timePtr)		/* Location to store time information. */
 {
-    (*tclGetTimeProcPtr) (timePtr, tclTimeClientData);
+    tclGetTimeProcPtr(timePtr, tclTimeClientData);
 }
 
 /*
@@ -414,7 +385,7 @@ NativeGetTime(
 
 		WaitForSingleObject(timeInfo.readyEvent, INFINITE);
 		CloseHandle(timeInfo.readyEvent);
-		Tcl_CreateExitHandler(StopCalibration, (ClientData) NULL);
+		Tcl_CreateExitHandler(StopCalibration, NULL);
 	    }
 	    timeInfo.initialized = TRUE;
 	}
@@ -515,93 +486,6 @@ StopCalibration(
     CloseHandle(timeInfo.calibrationThread);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclpGetTZName --
- *
- *	Gets the current timezone string.
- *
- * Results:
- *	Returns a pointer to a static string, or NULL on failure.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-char *
-TclpGetTZName(
-    int dst)
-{
-    int len;
-    char *zone, *p;
-    TIME_ZONE_INFORMATION tz;
-    Tcl_Encoding encoding;
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-    char *name = tsdPtr->tzName;
-
-    /*
-     * tzset() under Borland doesn't seem to set up tzname[] at all.
-     * tzset() under MSVC has the following weird observed behavior:
-     *	 First time we call "clock format [clock seconds] -format %Z -gmt 1"
-     *	 we get "GMT", but on all subsequent calls we get the current time
-     *	 ezone string, even though env(TZ) is GMT and the variable _timezone
-     *	 is 0.
-     */
-
-    name[0] = '\0';
-
-    zone = getenv("TZ");
-    if (zone != NULL) {
-	/*
-	 * TZ is of form "NST-4:30NDT", where "NST" would be the name of the
-	 * standard time zone for this area, "-4:30" is the offset from GMT in
-	 * hours, and "NDT is the name of the daylight savings time zone in
-	 * this area. The offset and DST strings are optional.
-	 */
-
-	len = strlen(zone);
-	if (len > 3) {
-	    len = 3;
-	}
-	if (dst != 0) {
-	    /*
-	     * Skip the offset string and get the DST string.
-	     */
-
-	    p = zone + len;
-	    p += strspn(p, "+-:0123456789");
-	    if (*p != '\0') {
-		zone = p;
-		len = strlen(zone);
-		if (len > 3) {
-		    len = 3;
-		}
-	    }
-	}
-	Tcl_ExternalToUtf(NULL, NULL, zone, len, 0, NULL, name,
-		sizeof(tsdPtr->tzName), NULL, NULL, NULL);
-    }
-    if (name[0] == '\0') {
-	if (GetTimeZoneInformation(&tz) == TIME_ZONE_ID_UNKNOWN) {
-	    /*
-	     * MSDN: On NT this is returned if DST is not used in the current
-	     * TZ
-	     */
-
-	    dst = 0;
-	}
-	encoding = Tcl_GetEncoding(NULL, "unicode");
-	Tcl_ExternalToUtf(NULL, encoding,
-		(char *) ((dst) ? tz.DaylightName : tz.StandardName), -1,
-		0, NULL, name, sizeof(tsdPtr->tzName), NULL, NULL, NULL);
-	Tcl_FreeEncoding(encoding);
-    }
-    return name;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -622,7 +506,7 @@ TclpGetTZName(
 
 struct tm *
 TclpGetDate(
-    CONST time_t *t,
+    const time_t *t,
     int useGMT)
 {
     struct tm *tmPtr;
@@ -1168,7 +1052,7 @@ AccumulateSample(
 
 struct tm *
 TclpGmtime(
-    CONST time_t *timePtr)	/* Pointer to the number of seconds since the
+    const time_t *timePtr)	/* Pointer to the number of seconds since the
 				 * local system's epoch */
 {
     /*
@@ -1199,9 +1083,8 @@ TclpGmtime(
 
 struct tm *
 TclpLocaltime(
-    CONST time_t *timePtr)	/* Pointer to the number of seconds since the
+    const time_t *timePtr)	/* Pointer to the number of seconds since the
 				 * local system's epoch */
-
 {
     /*
      * The MS implementation of localtime is thread safe because it returns
